"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const task_scheduler_1 = require("@microsoft/task-scheduler");
const npmTask_1 = require("./npmTask");
const filterPackages_1 = require("./filterPackages");
const logger_1 = require("../logger");
const workspace_tools_1 = require("workspace-tools");
/** Returns a map that maps task name to the priorities config for that task */
function getPriorityMap(priorities) {
    const result = new Map();
    priorities.forEach((entry) => {
        const taskPriority = result.get(entry.task) || {};
        taskPriority[entry.package] = entry.priority;
        result.set(entry.task, taskPriority);
    });
    return result;
}
async function runTasks(options) {
    const { graph, workspace, context, config } = options;
    const priorityMap = getPriorityMap(config.priorities);
    let pipeline = task_scheduler_1.createPipeline(graph, {
        // dummy logger for task-scheduler because lage already has logger for its tasks
        logger: {
            error: (_msg) => { },
            log: (_msg) => { },
        },
        exit: (code) => { },
        targetsOnly: config.only,
        concurrency: config.concurrency,
    });
    const taskNames = Object.keys(config.pipeline);
    for (const [task, taskDeps] of Object.entries(config.pipeline)) {
        const deps = taskDeps.filter((dep) => !dep.startsWith("^"));
        const topoDeps = taskDeps
            .filter((dep) => dep.startsWith("^"))
            .map((dep) => dep.slice(1));
        pipeline = pipeline.addTask({
            name: task,
            deps,
            topoDeps,
            priorities: priorityMap.get(task),
            run: async (_location, _stdout, _stderr, pkg) => {
                const scripts = workspace.allPackages[pkg].scripts;
                if (scripts && scripts[task]) {
                    return (await npmTask_1.npmTask(task, workspace.allPackages[pkg], config, context, workspace.root));
                }
                return true;
            },
        });
    }
    // Filter packages per --scope and command(s)
    const { scope, since } = config;
    // If scoped is defined, get scoped packages
    const hasScopes = Array.isArray(scope) && scope.length > 0;
    let scopedPackages = undefined;
    if (hasScopes) {
        scopedPackages = workspace_tools_1.getScopedPackages(scope, workspace.allPackages);
        scopedPackages = [
            ...scopedPackages,
            ...workspace_tools_1.getTransitiveProviders(scopedPackages, workspace.allPackages),
        ];
    }
    const hasSince = typeof since !== "undefined";
    let changedPackages = undefined;
    if (hasSince) {
        changedPackages = workspace_tools_1.getChangedPackages(workspace.root, since, config.ignore);
    }
    const filteredPackages = filterPackages_1.filterPackages({
        allPackages: workspace.allPackages,
        deps: config.deps,
        scopedPackages,
        changedPackages,
    });
    // Set up the longest names of tasks and scripts for nice logging
    logger_1.setTaskLogMaxLengths(Object.keys(workspace.allPackages).reduce((l, pkg) => (l < pkg.length ? pkg.length : l), 0), taskNames.reduce((l, task) => (l < task.length ? task.length : l), 0));
    await pipeline.go({
        packages: filteredPackages,
        tasks: config.command,
    });
}
exports.runTasks = runTasks;
